/*
 * cancan.h
 * various constants etc
 */

/* CONSTANTS */

#define CMDSEP ';'		/* command separator character */
#define DELIM " ;:.?!,'{}"	/* word delimiting characters */
#define MAXFDSCAN 32		/* max number of fd to scan in select() */
#define CAPLEN 20		/* max length of a terminal capability */
#define MAXARGS 16		/* max number of arguments to editor */
#define MAXSUBOPT 256		/* max length of suboption string */
#define BUFSIZE 2048		/* general i/o buffer size */
#define LINELEN 1024		/* maximum line length */
#define MAXHIST 128		/* number of history lines kept */
#define MAXMAPLEN 512		/* maximum length of automapped path */
#define WORDSKEPT 256		/* number of words kept for TAB-completion */
#define FLASHDELAY 500000	/* time of parentheses flash in microsecs */
#define KBD_TIMEOUT 35000	/* timeout for keyboard read in microsecs */

#define DEFAULT_EDITOR "emacs"	/* (environment) variable EDITOR overrides */
#define DEFAULT_PAGER "more"	/* (environment) variable PAGER overrides */

#define LM_NOECHO 1		/* no local echo */
#define LM_CHAR 2		/* char-by-char mode (no line editing) */

/* shared functions: */

/* from cancan.c */

void printver();
void exitcancan();
int execute();
void setsigs();
void intrsig();
void suspend_cancan();
int syswarn();
void syserr();
void check_prompt();
void save_settings();
void record_line();
int is_main_sess();
int main_sess();
void send_to_main_host();
int find_sess();
int connect_sess();
void disconnect_sess();
void snoop_sess();
void show_connections();

/* from parse.c */
void parse();
int parse_walk();
int search_action();
char *chop_arg();
void escape_command();
void escape_specials();
void unescape();
char *skipspace();
char *skipnum();

/* from mapping.c */
void remove_steps();
void retrace_steps();
void show_automap();
void add_dir_to_map();

/* from tel.c */
int doconnect();
int read_socket();
void send_term_size();
void send_to_host();
void flush_output_buffer();

/* from edit.c */
void key_run_command();
void put_word();

/* shared vars */

/* from cancan.c */
extern int mp_active;
extern int new_settings;	/* action/alias/etc has changed */
extern int verbose;		/* this is set if cmd is user-typed */
extern int wrap;
extern int speedwalk;
extern int verbatim;

extern int linemode;		/* line mode with flags as above (LM_*) */
extern int sockfd;		/* socket file descriptor */
extern int waiting_for_pid;
extern int wait_result;

extern int cols_1, lines_1;	/* for efficiency's sake */
extern int cols, lines;		/* terminal window size */
extern int line0, col0;		/* origin of input line */
extern char promptstr[];	/* prompt string */
extern char edbuf[];		/* input line buffer */
extern int edlen;		/* characters in edbuf */
extern int pos;			/* cursor position in edbuf */
extern int edattr;
extern char edattrbeg[], edattrend[];
extern char initstr[];

extern FILE *capfile;
extern char prefixstr[];
extern char inserted_next[];
extern char footnote[];

extern char *hist[];
extern int curline;
extern int pickline;

extern char *recording_name;

/* from tel.c */
extern int data_sent;		/* incremented each time we send_to_host */

/* from edit.c */
struct defbind_node {
  char label[8];
  char *seq, *name;
  void (*funct)();
};
extern struct defbind_node defbinds[];

/* ugly debug macro hack, vt100-like only */
#define NOTE(param) (printf("\0337\033[H\033[K"),printf param,\
		     printf("\0338"),fflush(stdout))

#ifdef STDASCII
/* ct_array defined in parse.c */
extern unsigned char ct_array[256];
#define TOLOWER(c) ((unsigned char)(c) | 0x20)
#define C_TYPE(c) (ct_array[(unsigned char)(c)])
#undef isprint
#undef isdigit
#define isprint(c) (C_TYPE(c) & 0x01)
#define isdigit(c) (C_TYPE(c) & 0x04)
#define isword(c)  (C_TYPE(c) & 0x0a)
#define islabel(c) (C_TYPE(c) & 0x0e)
#define isrparen(c) (C_TYPE(c) & 0x10)
#define islparen(c) (C_TYPE(c) & 0x20)
#define isplist(c) (C_TYPE(c) & 0x44)
#else
#define TOLOWER(c) (tolower(c))
#define isword(c) (isalpha(c) || (c) == '_')
#define islabel(c) (isalnum(c) || (c) == '_')
#define isrparen(c) ((c) == ')' || (c) == ']' || (c) == '}')
#define islparen(c) ((c) == '(' || (c) == '[' || (c) == '{')
#endif
/* the following two defines are also ASCII-dependant! */
#define ctoi(c) ((int)((unsigned char)(c) - '0'))
#define CONTROL(c) ((unsigned char)(c) & 0x1f)
#define LPAREN(c) ((c) == ')' ? '(' : ((c) == ']' ? '[' : '{'))

/* macros to find cursor position from input buffer position */
#define CURLINE(pos) (((pos) + col0) / cols_1 + line0)
#define CURCOL(pos) (((pos) + col0) % cols_1)

#define COLOURS 8		/* number of ANSI colours */
#define NOATTRIB COLOURS	/* no colour change, use default */
#define BOLD (COLOURS + 1)
#define UNDERLINE (COLOURS + 2)
#define INVERSE (COLOURS + 3)

/* constructors / accessors for attribute codes */
#define ATTRCODE(fg, bg) ((fg) | ((bg << 8)))
#define FOREGROUND(attrcode) ((attrcode) & 0xff)
#define BACKGROUND(attrcode) ((attrcode) >> 8)

#define NOATTRCODE ATTRCODE(NOATTRIB, NOATTRIB)

/*
 * NCSA telnet 2.2 doesn't reset the colour when it receives "esc [ m",
 * so we must know what the normal colours are in order to reset it.
 * These colours can be changed with the #stdcolour command.
 */
#ifdef TELNETBUG
#define DEFAULTFG 7		/* make default white text */
#define DEFAULTBG 4		/* on blue background */
#endif

#ifdef TERM
#define OMIT_SESSIONS
#endif

#ifdef OMIT_SESSIONS
#define send_to_main_host(buf, len) (send_to_host(buf, len))
#define main_sess() (sockfd)
#define is_main_sess() (1)
#endif
